from time import ctime
import os

from Framework import Controller
from Framework import templatepage
from Framework.PluginManager import DeclarePlugin

from cherrypy import expose
from cherrypy import request

from dataprovider.runDataProviders import *
from dataprovider.jobDataProviders import *
from dataprovider.fileDataProviders import *

from guiProvider import *
from utils import *
from consts import *

import globals
import config
import sqlManager

# Table IDs
RUNS_T = 'runsT'
REPACKER_T = 'repackerT'
REPACK_S_T = 'repackST'
MERGE_T = 'mergeT'
MERGE_S_T = 'mergeST'
RECO_T = 'recoT'
RECO_S_T = 'recoST'
ALCASKIM_T = 'alcaskimT'
ALCASKIM_S_T = 'alcaskimST'
RUN_REPACK_T = 'runRepackT'
RUN_CONFIG_KEY_T = 'runConfigKeyT'
RUN_REPACK_MERGE_T = 'runRepackMergeT'
RUN_RECO_T = 'runRecoT'
RUN_RECO_MERGE_T = 'runRecoMergeT'
RUN_ALCASKIM_T = 'runAlcaskimT'
RUN_ALCASKIM_MERGE_T = 'runAlcaskimMergeT'
RUN_STREAMER_T = 'runStreamerT'
RUN_REPACK_STATS_T = 'runRepackStatsT'
RUN_MERGE_STATS_T = 'runMergeStatsT'
RUN_RECO_STATS_T = 'runRecoStatsT'
RUN_RECO_MERGE_STATS_T = 'runRecoMergeStatsT'
RUN_ALCASKIM_STATS_T = 'runAlcaskimStatsT'
#RUN_ALCASKIM_MERGE_STATS_T = 'runAlcaskimMergeStatsT'

T_STREAM_FILES = 'streamerFiles'
T_RECO_FILES = 'recoFiles'
T_RECO_JOBS = 'recoJobs'
T_REPACK_FILES = 'repackFiles'
T_REPACK_JOBS = 'repackJobs'
T_MERGE_FILES = 'mergeFiles'
T_MERGE_JOBS = 'mergeJobs'

T_JOBS = 'jobs'

# Identify cheetah template full path
TMPL_PATH = os.path.join(__file__.rsplit ("/", 1)[0], "Templates/")

class T0Mon (Controller):
    """
    Web page controller
    Provides GUI for user and communicates with backend
    User input should be dealt with only in this class
    """

    def __init__(self, context):
        """
        Init the controller
        """
        Controller.__init__(self, context, __file__)
        # get logger from context
        globals.logger = self.context.Logger()
        # connect to DB
        sqlManager.initBasicConnection()

    #### Public ####

    @expose
    def repackJob(self, id, **kwds):
        """
        Display repack job history
        """
        globals.pageParams = kwds
        entity = 'Repack job'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackData = RepackJobByIdsDP([id], params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
            repackJobs = wrapInBorder(repackJobs)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            jobIds = [int(line[0]) for line in repackData.data]
            repackData = RepackByRepackJobIdsDP(jobIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            mergeData = RepackMergeJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            mergeData = RepackMergeByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            mergeIds = [int(line[0]) for line in mergeData.data]
            recoData = RecoJobByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoIds = [int(line[0]) for line in recoData.data]
            recoData = RecoByRecoJobIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def reco(self, id, **kwds):
        """
        Display reco file history
        """
        globals.pageParams = kwds
        entity = 'Reco file'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoData = RecoByIdsDP([id], params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
            recoFiles = wrapInBorder(recoFiles)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoIds = [int(line[0]) for line in recoData.data]
            recoData = RecoJobByRecoIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            recoJobIds = [int(line[0]) for line in recoData.data]
            mergeFilesData = RepackMergeByRecoJobIdsDP(recoJobIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeJobIds = [int(line[0]) for line in mergeFilesData.data]
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeJobIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            filesIds = [int(line[0]) for line in mergeFilesData.data]
            repackData = RepackByRepackMergeIdsDP(filesIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def recoJob(self, id, **kwds):
        """
        Display reco job history
        """
        globals.pageParams = kwds
        entity = 'Reco job'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_RECO_JOBS)
            recoJobData = RecoJobByIdsDP([id], params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
            recoJobs = wrapInBorder(recoJobs)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoIds = [int(line[0]) for line in recoJobData.data]
            recoData = RecoByRecoJobIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            recoJobIds = [int(line[0]) for line in recoJobData.data]
            mergeFilesData = RepackMergeByRecoJobIdsDP(recoJobIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeJobIds = [int(line[0]) for line in mergeFilesData.data]
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeJobIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            filesIds = [int(line[0]) for line in mergeFilesData.data]
            repackData = RepackByRepackMergeIdsDP(filesIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def repackMerge(self, id, **kwds):
        """
        Display repack merge file history
        """
        globals.pageParams = kwds
        entity = 'Repack merge file'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_MERGE_FILES)
            mergeFilesData = RepackMergeByIdsDP([id], params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
            mergeFiles = wrapInBorder(mergeFiles)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            recoJobData = RecoJobByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoIds = [int(line[0]) for line in recoJobData.data]
            recoData = RecoByRecoJobIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeJobIds = [int(line[0]) for line in mergeFilesData.data]
            mergeData = RepackMergeJobByRepackMergeIdsDP(mergeJobIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            filesIds = [int(line[0]) for line in mergeFilesData.data]
            repackData = RepackByRepackMergeIdsDP(filesIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def repackMergeJob(self, id, **kwds):
        """
        Display repack merge job history
        """
        globals.pageParams = kwds
        entity = 'Repack merge job'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            mergeData = RepackMergeJobByIdsDP([id], params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
            mergeJobs = wrapInBorder(mergeJobs)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            mergeJobIds = [int(line[0]) for line in mergeData.data]
            mergeFilesData = RepackMergeByRepackMergeJobIdsDP(mergeJobIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeFilesData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            mergeIds = [int(line[0]) for line in mergeFilesData.data]
            recoJobData = RecoJobByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoJobData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoIds = [int(line[0]) for line in recoJobData.data]
            recoData = RecoByRecoJobIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            filesIds = [int(line[0]) for line in mergeFilesData.data]
            repackData = RepackByRepackMergeIdsDP(filesIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def streamer(self, id, **kwds):
        """
        Display streamer file history
        """
        globals.pageParams = kwds
        entity = "Streamer file"

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_STREAM_FILES)
            streamerData = StreamerByIdsDP([id], params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
            streamerFiles = wrapInBorder(streamerFiles)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            streamerIds = [int(line[0]) for line in streamerData.data]
            repackData = RepackJobByStreamerDP(streamerIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_FILES)
            jobIds = [int(line[0]) for line in repackData.data]
            repackData = RepackByRepackJobIdsDP(jobIds, params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            mergeData = RepackMergeJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            mergeData = RepackMergeByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            mergeIds = [int(line[0]) for line in mergeData.data]
            recoData = RecoJobByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoIds = [int(line[0]) for line in recoData.data]
            recoData = RecoByRecoJobIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def repack(self, id, **kwds):
        """
        Display repack file history
        """
        globals.pageParams = kwds
        entity = 'Repacked file'

        try:
            id = int(id)
        except ValueError:
            return self.templatePage (
                "error", {
                    'msg': "Bad " + entity + " id",
                    "tmplPath" : TMPL_PATH
                }
            )

        try:
            params = self._getTableParams(T_REPACK_FILES)
            repackData = RepackByIdsDP([id], params.orderby, params.asc, params.page)
            repackFiles = getTableGui(T_REPACK_FILES, repackData)
            repackFiles = wrapInBorder(repackFiles)
        except: repackFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_REPACK_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            repackJobData = RepackJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            repackJobs = getTableGui(T_REPACK_JOBS, repackJobData)
        except: repackJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_STREAM_FILES)
            repackJobIds = [int(line[0]) for line in repackJobData.data]
            streamerData = StreamerByRepackJobIdsDP(repackJobIds, params.orderby, params.asc, params.page)
            streamerFiles = getTableGui(T_STREAM_FILES, streamerData)
        except: streamerFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_JOBS)
            repackIds = [int(line[0]) for line in repackData.data]
            mergeData = RepackMergeJobByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeJobs = getTableGui(T_MERGE_JOBS, mergeData)
        except: mergeJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_MERGE_FILES)
            mergeData = RepackMergeByRepackIdsDP(repackIds, params.orderby, params.asc, params.page)
            mergeFiles = getTableGui(T_MERGE_FILES, mergeData)
        except: mergeFiles = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_JOBS)
            mergeIds = [int(line[0]) for line in mergeData.data]
            recoData = RecoJobByMergeIdsDP(mergeIds, params.orderby, params.asc, params.page)
            recoJobs = getTableGui(T_RECO_JOBS, recoData)
        except:
            recoJobs = formatExceptionInfo()

        try:
            params = self._getTableParams(T_RECO_FILES)
            recoIds = [int(line[0]) for line in recoData.data]
            recoData = RecoByRecoJobIdsDP(recoIds, params.orderby, params.asc, params.page)
            recoFiles = getTableGui(T_RECO_FILES, recoData)
        except:
            recoFiles = formatExceptionInfo()

        return self.templatePage (
                "history", {
                    T_STREAM_FILES: streamerFiles,
                    T_REPACK_JOBS : repackJobs,
                    T_REPACK_FILES : repackFiles,
                    T_MERGE_JOBS : mergeJobs,
                    T_MERGE_FILES : mergeFiles,
                    T_RECO_JOBS : recoJobs,
                    T_RECO_FILES : recoFiles,
                    'entity' : entity + ' ' + str(id),
                    "tmplPath" : TMPL_PATH
                }
            )

    @expose
    def jobs(self, **kwds):
        """
        Display jobs for particular run and/or status
        """
        globals.pageParams = kwds
        error = ""
        table = ""
        if globals.pageParams.has_key('table'):
            table = str(kwds['table'])
        else: error += "<br />No table name specified"
        if globals.pageParams.has_key('status'):
            try:
                status = int(kwds['status'])
            except ValueError:
                error += "<br />Status is not an int"
        else: error += "<br />No status flag specified"
        if globals.pageParams.has_key('runid'):
            try:
                runid = int(kwds['runid'])
            except ValueError:
                error += "<br />Run id is not an int"
        else: runid = None

        params = self._getTableParams(T_JOBS)

        if table != "" and error == "":
            if table == TABLES.REPACK_JOBS:
                if runid != None:
                    jtd = RepackJobByStatusAndRunIdDP(status, runid, params.orderby, params.asc, params.page)
                else:
                    jtd = RepackJobByStatusDP(status, params.orderby, params.asc, params.page)
            elif table == TABLES.RECO_JOBS:
                if runid != None:
                    jtd = RecoJobByStatusAndRunIdDP(status, runid, params.orderby, params.asc, params.page)
                else:
                    jtd = RecoJobByStatusDP(status, params.orderby, params.asc, params.page)
            elif table == TABLES.MERGE_JOBS:
                if runid != None:
                    jtd = RepackMergeJobByStatusAndRunIdDP(status, runid, params.orderby, params.asc, params.page)
                else:
                    jtd = RepackMergeJobByStatusDP(status, params.orderby, params.asc, params.page)
            else: error += "<br />Bad jobs table name specified"

        if error == "":
            jt = getTableGui(T_JOBS, jtd)
        else:
            jt = error

        return self.templatePage("jobs", {
            T_JOBS: jt,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    @expose
    def run(self, runid, **kwds):
        """
        Display run statistics
        """
        globals.pageParams = kwds

        try:
            runid = int(runid)
        except ValueError:
            return self.templatePage ("error", {
                'msg': formatExceptionInfo(),
                "tmplPath" : TMPL_PATH
            })

        params = self._getTableParams(RUN_REPACK_MERGE_T)

        try:
            rrmtd = RepackMergeByRunIdDP(runid, params.orderby, params.asc, params.page)
            rrmt = getTableGui(RUN_REPACK_MERGE_T, rrmtd)
        except: rrmt = formatExceptionInfo()

        params = self._getTableParams(RUN_REPACK_T)

        try:
            rrtd = RepackByRunIdDP(runid, params.orderby, params.asc, params.page)
            rrt = getTableGui(RUN_REPACK_T, rrtd)
        except: rrt = formatExceptionInfo()

        params = self._getTableParams(RUN_RECO_T)

        try:
            rrtd = RecoByRunIdDP(runid, params.orderby, params.asc, params.page)
            rret = getTableGui(RUN_RECO_T, rrtd)
        except: rret = formatExceptionInfo()

        params = self._getTableParams(RUN_STREAMER_T)

        try:
            rstd = StreamerByRunIdDP(runid, params.orderby, params.asc, params.page)
            rst = getTableGui(RUN_STREAMER_T, rstd)
        except: rst = formatExceptionInfo()

	params = self._getTableParams(RUN_RECO_MERGE_T)

        try:
            rrecomtd = RecoMergeByRunIdDP(runid, params.orderby, params.asc, params.page)
            rrecomt = getTableGui(RUN_RECO_MERGE_T, rrecomtd)
        except: rrecomt = formatExceptionInfo()

        params = self._getTableParams(RUN_ALCASKIM_T)

        try:
            ratd = AlcaskimByRunIdDP(runid, params.orderby, params.asc, params.page)
            rat = getTableGui(RUN_ALCASKIM_T, ratd)
        except: rat = formatExceptionInfo()

        params = self._getTableParams(RUN_ALCASKIM_MERGE_T)

        try:
            ramtd = AlcaskimMergeByRunIdDP(runid, params.orderby, params.asc, params.page)
            ramt = getTableGui(RUN_ALCASKIM_MERGE_T, ramtd)
        except: ramt = formatExceptionInfo()

        try:
            rrstd = RepackStatsDataProvider(runid)
            rrst = getVertTableGui(RUN_REPACK_T, rrstd)
        except: rrst = formatExceptionInfo()

        try:
            rmstd = JobsStatsDataProvider("Merge", "Merge stats", runid)
	    rmstd.load()
            rmst = getVertTableGui(RUN_REPACK_T, rmstd)
        except: rmst = formatExceptionInfo()

        try:
            rcstd = JobsStatsDataProvider("PromptReco", "Reco stats", runid)
	    rcstd.load()
            rcst = getVertTableGui(RUN_REPACK_T, rcstd)
        except: rcst = formatExceptionInfo()

#        try:
#            rrmstd = RecoMergeStatsDataProvider(runid)
#            rrmst = getVertTableGui(RUN_REPACK_T, rrmstd)
#        except: rrmst = formatExceptionInfo()

	rrmst = ""

        try:
            rastd = JobsStatsDataProvider("AlcaSkim", "Alcaskim stats", runid)
            rastd.load()
            rast = getVertTableGui(RUN_REPACK_T, rastd)
        except: rast = formatExceptionInfo()

        try:
            rdtd = RunConfigKeyProvider(runid)
            rdt = getDoubleTableGui(RUN_CONFIG_KEY_T, rdtd)
        except: rdt = formatExceptionInfo()

        return self.templatePage("run", {
            RUN_REPACK_T : rrt,
            RUN_RECO_T : rret,
            RUN_RECO_MERGE_T : rrecomt,
            RUN_ALCASKIM_T : rat,
            RUN_ALCASKIM_MERGE_T : ramt,
            RUN_STREAMER_T : rst,
            RUN_REPACK_STATS_T : rrst,
            RUN_MERGE_STATS_T : rmst,
            RUN_RECO_STATS_T : rcst,
	    RUN_RECO_MERGE_STATS_T : rrmst, 
	    RUN_ALCASKIM_STATS_T : rast,
            RUN_REPACK_MERGE_T : rrmt,
            RUN_CONFIG_KEY_T : rdt,
            #"streamRepackG" : srg,
            "runid" : runid,
            "time" : ctime(),
            "tmplPath" : TMPL_PATH
        })

    @expose
    def index(self, **kwds):
        """
        Display last runs, job statistics and running jobs
        """

        globals.pageParams = kwds

        #### Runs table ####

        params = self._getTableParams(RUNS_T)

        try:
            runsTableTd = RunsDataProvider(params.orderby, params.asc, params.page)
            runsTable = getTableGui(RUNS_T, runsTableTd, 100)
        except: runsTable = formatExceptionInfo()

        #### Repacking status table ####

        params = self._getTableParams(REPACKER_T)

        try:
            repackerTd = RepackRunningJobDP(params.orderby, params.asc, params.page)
            repackerTable = getTableGui(REPACKER_T, repackerTd)
        except: repackerTable = formatExceptionInfo()

        #### Reconstruction status table ####

        params = self._getTableParams(RECO_T)

        try:
            recoTd = RecoRunningJobDP(params.orderby, params.asc, params.page)
            recoTable = getTableGui(RECO_T, recoTd)
        except: recoTable = formatExceptionInfo()

        #### Merging status table ####

        params = self._getTableParams(MERGE_T)

        try:
            mergeTd = RepackMergeRunningJobDP(params.orderby, params.asc, params.page)
            mergeTable = getTableGui(MERGE_T, mergeTd)
        except: mergeTable = formatExceptionInfo()

        #### Alcaskim status table ####

        params = self._getTableParams(ALCASKIM_T)

        try:
            alcaskimTd = AlcaskimRunningJobDP(params.orderby, params.asc, params.page)
            alcaskimTable = getTableGui(ALCASKIM_T, alcaskimTd)
        except: alcaskimTable = formatExceptionInfo()

        try:
            repackStatsTd = StatsDataProvider("Repack stats", TABLES.REPACK_JOBS)
            repackStatsTd.load()
            repackStatsTable = getVertTableGui(REPACK_S_T, repackStatsTd)
        except: repackStatsTable = formatExceptionInfo()

        try:
            recoStatsTd = JobsStatsDataProvider("PromptReco", "Reco stats")
            recoStatsTd.load()
            recoStatsTable = getVertTableGui(RECO_S_T, recoStatsTd)
        except: recoStatsTable = formatExceptionInfo()

        try:
            #mergeStatsTd = StatsDataProvider("Merge stats", TABLES.MERGE_JOBS)
	    mergeStatsTd = JobsStatsDataProvider("Merge", "Merge stats")
            mergeStatsTd.load()
            mergeStatsTable = getVertTableGui(MERGE_S_T, mergeStatsTd)
        except: mergeStatsTable = formatExceptionInfo()

        try:
            alcaskimStatsTd = JobsStatsDataProvider("Alcaskim", "Alcaskim stats")
            alcaskimStatsTd.load()
            alcaskimStatsTable = getVertTableGui(ALCASKIM_S_T, alcaskimStatsTd)
        except: alcaskimStatsTable = formatExceptionInfo()

        url = request.base
        if DEV_SERVER in url: dev = True
        else: dev = False

        return self.templatePage("index", {
            RUNS_T : runsTable,
            REPACKER_T : repackerTable,
            RECO_T : recoTable,
	    ALCASKIM_T : alcaskimTable, 
            MERGE_T : mergeTable,
            MERGE_S_T : mergeStatsTable,
            REPACK_S_T : repackStatsTable,
            RECO_S_T : recoStatsTable,
	    ALCASKIM_S_T : alcaskimStatsTable,
            "time" : ctime(),
            "dev" : dev,
            "tmplPath" : TMPL_PATH
        })

    #### Private ####

    def _getTableParams(self, tableid, orderby = 0, asc = False, page = 0):
        """
        Parse table params from URL
        """
        if globals.pageParams.has_key(tableid + "_p"):
            try:
                pages = globals.pageParams[tableid + "_p"]
                if pages == "all": page = None
                else:
                    page = int(globals.pageParams[tableid + "_p"])
                    if page < 0:
                        page = 0
                        self.context.Logger().error("Page num < 0")
            except ValueError:
                self.context.Logger().error("Could not convert page num to an int")

        if globals.pageParams.has_key(tableid):
            words = globals.pageParams[tableid].rsplit("_")
            try:
                orderby = int(words[0])
                if orderby < 0:
                    orderby = 0
                    self.context.Logger().error("Col id < 0")

            except ValueError:
                self.context.Logger().error("Could not convert table row id to an int")

            if (len(words) == 2):
                if (words[1] == "True"):
                    asc = True
                else: asc = False
            else:
                self.context.Logger().error("Badly formed table params")

        params = TableParams(orderby, asc, page)

        return params

class TableParams:
    """
    Table parameters
    """
    def __init__(self, orderby = 0, asc = False, page = 0):
        """
        Parameters:

        orderby    table ordering column id
        asc        order ascending or descending?
        page       page number to display
        """
        self.orderby = orderby
        self.asc = asc
        self.page = page

# Declare controllers
DeclarePlugin ("/Controllers/" + PROJECT_NAME + "/" + PROJECT_NAME, T0Mon, {"baseUrl": "/" + PROJECT_NAME})

