from sqlalchemy import *

import config

from globals import logger

from utils import *
from consts import *
from sqlManager import *

### Analysis module ###
# This module is for functions used for
# analysing table data after executing main query

def getRunRepackedPercentage(runid, numStreamers):
    """
    Get how many percent streamers files were repacked
    """
    assocTable = Tbl(TABLES.REPACK_STREAMER_ASSOC, True)
    streamerTable = Tbl(TABLES.STREAMER, True)

    query = """ SELECT count(distinct a.streamer_id)
		FROM """ + assocTable  + """ a INNER JOIN """ + streamerTable + """ s ON (a.streamer_id = s.streamer_id)
		WHERE s.run_id = """ + str(runid)

    result = executeQuery(query)
    numStreamersUsedToRepack = int(result.fetchone()[0])

    if (numStreamers == 0):
        percentage = 100
    else:
        percentage = numStreamersUsedToRepack * 100 / numStreamers
    return Percentage(percentage, numStreamersUsedToRepack)

def getRunExportedRepackMergePercentage(runid, numRepacked, numExportedRepackMerge):
    """
    Get how many percent repacked streamers files were exported
    """
    repackMerge = getRunRepackMergePercentage(runid, numRepacked)
    
    numRepackMerge = repackMerge.value
    
    if (numRepacked == 0):
	percentage = 100
    elif (numRepackMerge == 0):
	percentage = 0
    else:
        percentage = numExportedRepackMerge * 100 / numRepackMerge

    return Percentage(percentage, numExportedRepackMerge)

def getRunRepackMergePercentage(runid, numRepacked):
    """
    Get how many percent repacked files were merged
    """
    wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
    wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
    datasetTable = Tbl(TABLES.DATASET_PATH, True)
    dataTierTable = Tbl(TABLES.DATA_TIER, True)
    wmbsFileRunumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
    wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
    wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
    wmbsSubsTypeTable = Tbl(TABLES.WMBS_SUBS_TYPE, True)

    #### TODO : Review this query. It should return how many repacked files have been used to make merged files, that means how many repacked files were merged

    query = """ SELECT COUNT(DISTINCT f.id)
		FROM """ + wmbsFileDetailsTable + """ f  INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                        INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                        INNER JOIN """ + dataTierTable + """ dt ON (dp.data_tier = dt.id)
                        INNER JOIN """ + wmbsFileRunumiMapTable + """ m ON (f.id = m.fileid)
                        INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
		WHERE dt.name = 'RAW' and m.run = """ + str(runid) + """ and c.subscription = 
			(SELECT s.id 
			 FROM """ + wmbsSubscriptionTable + """ s INNER JOIN """ + wmbsSubsTypeTable + """ t ON (s.subtype = t.id)
	                 WHERE t.name = 'Mergeable')"""

    result = executeQuery(query)
    numMerged = int(result.fetchone()[0])

    if numRepacked == 0:
        percentage = 100
    else:
        percentage = numMerged * 100 / numRepacked

    return Percentage(percentage, numMerged)

def getRunRecoPercentage(runid, numReconstructed):
    """
    Get how many merged files, marked for reconstruction were reconstructed
    """
    wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
    wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
    datasetTable = Tbl(TABLES.DATASET_PATH, True)
    dataTierTable = Tbl(TABLES.DATA_TIER, True)
    wmbsFileRunumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
    wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
    wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)

    query = """ SELECT COUNT(DISTINCT f.id)
                FROM """ + wmbsFileDetailsTable + """ f 
                        INNER JOIN """ + wmbsFileRunumiMapTable + """ m ON (f.id = m.fileid)
                        INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                        INNER JOIN """ + wmbsFilesetTable + """ fs ON (fsf.fileset = fs.id)
                WHERE m.run = """ + str(runid) + """ AND fs.name = 'Reconstructable' """

    result = executeQuery(query)
    numMarkedForReconstruction = int(result.fetchone()[0])

    if (numMarkedForReconstruction == 0):
        percentage = 100
    else:
        percentage = numReconstructed * 100 / numMarkedForReconstruction
    return Percentage(percentage, numReconstructed)

def getRunRecoExportedPercentage(runid, numReconstructed, numExported):
    """
    Get how many merged files, marked for reconstruction were already reconstructed and also exported
    """

    if (numReconstructed == 0):
        percentage = 100
    else:
        percentage = numExported * 100 / numReconstructed
    return Percentage(percentage, numExported)

def getRunAlcaskimPercentage(runid, numAlcaskimmed):
    """
    Get how many files, marked to be alcaskimmed, were actually alcaskimmed
    """
    wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
    wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
    datasetTable = Tbl(TABLES.DATASET_PATH, True)
    dataTierTable = Tbl(TABLES.DATA_TIER, True)
    wmbsFileRunumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
    wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
    wmbsFilesetFilesTable = Tbl(TABLES.WMBS_FILESET_FILES, True)

    query = """ SELECT COUNT(DISTINCT f.id)
                FROM """ + wmbsFileDetailsTable + """ f
                        INNER JOIN """ + wmbsFileRunumiMapTable + """ m ON (f.id = m.fileid)
                        INNER JOIN """ + wmbsFilesetFilesTable + """ fsf ON (f.id = fsf.fileid)
                        INNER JOIN """ + wmbsFilesetTable + """ fs ON (fsf.fileset = fs.id)
                WHERE m.run = """ + str(runid) + """ AND fs.name = 'AlcaSkimmable' """

    result = executeQuery(query)
    numMarkedForAlcaskim = int(result.fetchone()[0])

    if (numMarkedForAlcaskim == 0):
        percentage = 100
    else:
        percentage = numAlcaskimmed * 100 / numMarkedForAlcaskim
    return Percentage(percentage, numAlcaskimmed)

def getRunAlcaskimExportedPercentage(runid, numAlcaskimmed, numExported):
    """
    Get how many alcaskimmed files, have already been exported
    """

    if (numAlcaskimmed == 0):
        percentage = 100
    else:
        percentage = numExported * 100 / numAlcaskimmed
    return Percentage(percentage, numExported)


class Percentage:
    def __init__(self, percentage, value = None, total = None):
        """
        Parameters:

        percentage -    percentage
        value -         value for percentage
        total -         how much is 100%
        """
        self.percentage = percentage
        self.value = value
        self.total = total
